# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class ResourceTagRecord(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'organization_id': 'str',
        'resource_type': 'ResourceTagResourceType',
        'resource_id': 'str',
        'key': 'str',
        'value': 'str',
        'created_at': 'datetime',
        'updated_at': 'datetime'
    }

    attribute_map = {
        'id': 'id',
        'organization_id': 'organization_id',
        'resource_type': 'resource_type',
        'resource_id': 'resource_id',
        'key': 'key',
        'value': 'value',
        'created_at': 'created_at',
        'updated_at': 'updated_at'
    }

    def __init__(self, id=None, organization_id=None, resource_type=None, resource_id=None, key=None, value=None, created_at=None, updated_at=None, local_vars_configuration=None):  # noqa: E501
        """ResourceTagRecord - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._organization_id = None
        self._resource_type = None
        self._resource_id = None
        self._key = None
        self._value = None
        self._created_at = None
        self._updated_at = None
        self.discriminator = None

        self.id = id
        self.organization_id = organization_id
        self.resource_type = resource_type
        self.resource_id = resource_id
        self.key = key
        self.value = value
        self.created_at = created_at
        self.updated_at = updated_at

    @property
    def id(self):
        """Gets the id of this ResourceTagRecord.  # noqa: E501


        :return: The id of this ResourceTagRecord.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ResourceTagRecord.


        :param id: The id of this ResourceTagRecord.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def organization_id(self):
        """Gets the organization_id of this ResourceTagRecord.  # noqa: E501


        :return: The organization_id of this ResourceTagRecord.  # noqa: E501
        :rtype: str
        """
        return self._organization_id

    @organization_id.setter
    def organization_id(self, organization_id):
        """Sets the organization_id of this ResourceTagRecord.


        :param organization_id: The organization_id of this ResourceTagRecord.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and organization_id is None:  # noqa: E501
            raise ValueError("Invalid value for `organization_id`, must not be `None`")  # noqa: E501

        self._organization_id = organization_id

    @property
    def resource_type(self):
        """Gets the resource_type of this ResourceTagRecord.  # noqa: E501


        :return: The resource_type of this ResourceTagRecord.  # noqa: E501
        :rtype: ResourceTagResourceType
        """
        return self._resource_type

    @resource_type.setter
    def resource_type(self, resource_type):
        """Sets the resource_type of this ResourceTagRecord.


        :param resource_type: The resource_type of this ResourceTagRecord.  # noqa: E501
        :type: ResourceTagResourceType
        """
        if self.local_vars_configuration.client_side_validation and resource_type is None:  # noqa: E501
            raise ValueError("Invalid value for `resource_type`, must not be `None`")  # noqa: E501

        self._resource_type = resource_type

    @property
    def resource_id(self):
        """Gets the resource_id of this ResourceTagRecord.  # noqa: E501


        :return: The resource_id of this ResourceTagRecord.  # noqa: E501
        :rtype: str
        """
        return self._resource_id

    @resource_id.setter
    def resource_id(self, resource_id):
        """Sets the resource_id of this ResourceTagRecord.


        :param resource_id: The resource_id of this ResourceTagRecord.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and resource_id is None:  # noqa: E501
            raise ValueError("Invalid value for `resource_id`, must not be `None`")  # noqa: E501

        self._resource_id = resource_id

    @property
    def key(self):
        """Gets the key of this ResourceTagRecord.  # noqa: E501


        :return: The key of this ResourceTagRecord.  # noqa: E501
        :rtype: str
        """
        return self._key

    @key.setter
    def key(self, key):
        """Sets the key of this ResourceTagRecord.


        :param key: The key of this ResourceTagRecord.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and key is None:  # noqa: E501
            raise ValueError("Invalid value for `key`, must not be `None`")  # noqa: E501

        self._key = key

    @property
    def value(self):
        """Gets the value of this ResourceTagRecord.  # noqa: E501


        :return: The value of this ResourceTagRecord.  # noqa: E501
        :rtype: str
        """
        return self._value

    @value.setter
    def value(self, value):
        """Sets the value of this ResourceTagRecord.


        :param value: The value of this ResourceTagRecord.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and value is None:  # noqa: E501
            raise ValueError("Invalid value for `value`, must not be `None`")  # noqa: E501

        self._value = value

    @property
    def created_at(self):
        """Gets the created_at of this ResourceTagRecord.  # noqa: E501


        :return: The created_at of this ResourceTagRecord.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this ResourceTagRecord.


        :param created_at: The created_at of this ResourceTagRecord.  # noqa: E501
        :type: datetime
        """
        if self.local_vars_configuration.client_side_validation and created_at is None:  # noqa: E501
            raise ValueError("Invalid value for `created_at`, must not be `None`")  # noqa: E501

        self._created_at = created_at

    @property
    def updated_at(self):
        """Gets the updated_at of this ResourceTagRecord.  # noqa: E501


        :return: The updated_at of this ResourceTagRecord.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this ResourceTagRecord.


        :param updated_at: The updated_at of this ResourceTagRecord.  # noqa: E501
        :type: datetime
        """
        if self.local_vars_configuration.client_side_validation and updated_at is None:  # noqa: E501
            raise ValueError("Invalid value for `updated_at`, must not be `None`")  # noqa: E501

        self._updated_at = updated_at

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ResourceTagRecord):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ResourceTagRecord):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class TaskExceptionGroupAggregate(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'exception_type': 'str',
        'ray_session_name': 'str',
        'metrics': 'TaskGroupedAggregateMetrics'
    }

    attribute_map = {
        'exception_type': 'exception_type',
        'ray_session_name': 'ray_session_name',
        'metrics': 'metrics'
    }

    def __init__(self, exception_type=None, ray_session_name=None, metrics=None, local_vars_configuration=None):  # noqa: E501
        """TaskExceptionGroupAggregate - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._exception_type = None
        self._ray_session_name = None
        self._metrics = None
        self.discriminator = None

        self.exception_type = exception_type
        self.ray_session_name = ray_session_name
        self.metrics = metrics

    @property
    def exception_type(self):
        """Gets the exception_type of this TaskExceptionGroupAggregate.  # noqa: E501


        :return: The exception_type of this TaskExceptionGroupAggregate.  # noqa: E501
        :rtype: str
        """
        return self._exception_type

    @exception_type.setter
    def exception_type(self, exception_type):
        """Sets the exception_type of this TaskExceptionGroupAggregate.


        :param exception_type: The exception_type of this TaskExceptionGroupAggregate.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and exception_type is None:  # noqa: E501
            raise ValueError("Invalid value for `exception_type`, must not be `None`")  # noqa: E501

        self._exception_type = exception_type

    @property
    def ray_session_name(self):
        """Gets the ray_session_name of this TaskExceptionGroupAggregate.  # noqa: E501


        :return: The ray_session_name of this TaskExceptionGroupAggregate.  # noqa: E501
        :rtype: str
        """
        return self._ray_session_name

    @ray_session_name.setter
    def ray_session_name(self, ray_session_name):
        """Sets the ray_session_name of this TaskExceptionGroupAggregate.


        :param ray_session_name: The ray_session_name of this TaskExceptionGroupAggregate.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and ray_session_name is None:  # noqa: E501
            raise ValueError("Invalid value for `ray_session_name`, must not be `None`")  # noqa: E501

        self._ray_session_name = ray_session_name

    @property
    def metrics(self):
        """Gets the metrics of this TaskExceptionGroupAggregate.  # noqa: E501


        :return: The metrics of this TaskExceptionGroupAggregate.  # noqa: E501
        :rtype: TaskGroupedAggregateMetrics
        """
        return self._metrics

    @metrics.setter
    def metrics(self, metrics):
        """Sets the metrics of this TaskExceptionGroupAggregate.


        :param metrics: The metrics of this TaskExceptionGroupAggregate.  # noqa: E501
        :type: TaskGroupedAggregateMetrics
        """
        if self.local_vars_configuration.client_side_validation and metrics is None:  # noqa: E501
            raise ValueError("Invalid value for `metrics`, must not be `None`")  # noqa: E501

        self._metrics = metrics

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, TaskExceptionGroupAggregate):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, TaskExceptionGroupAggregate):
            return True

        return self.to_dict() != other.to_dict()



from contextlib import suppress
import os
from types import SimpleNamespace
from typing import cast, List, Optional

from anyscale._private.anyscale_client.common import AnyscaleClientInterface
from anyscale._private.models.image_uri import ImageURI
from anyscale._private.models.model_base import ResultIterator
from anyscale._private.sdk.base_sdk import BaseSDK
from anyscale.cli_logger import BlockLogger
from anyscale.client.openapi_client.models.decorated_application_template import (
    DecoratedApplicationTemplate,
)
from anyscale.image.models import ImageBuild, ImageBuildStatus
from anyscale.sdk.anyscale_client import (
    ClusterEnvironmentBuild,
    ClusterEnvironmentBuildStatus,
)


ENABLE_IMAGE_BUILD_FOR_TRACKED_REQUIREMENTS = (
    os.environ.get("ANYSCALE_ENABLE_IMAGE_BUILD_FOR_TRACKED_REQUIREMENTS", "0") == "1"
)


MAX_PAGE_SIZE = 50


class PrivateImageSDK(BaseSDK):
    def __init__(
        self,
        *,
        logger: Optional[BlockLogger] = None,
        client: Optional[AnyscaleClientInterface] = None,
    ):
        super().__init__(logger=logger, client=client)
        self._enable_image_build_for_tracked_requirements = (
            ENABLE_IMAGE_BUILD_FOR_TRACKED_REQUIREMENTS
        )

    @property
    def enable_image_build_for_tracked_requirements(self) -> bool:
        return self._enable_image_build_for_tracked_requirements

    def get_default_image(self) -> str:
        return self.client.get_default_build_id()

    def get_image_build(self, build_id: str) -> Optional[ClusterEnvironmentBuild]:
        return self.client.get_cluster_env_build(build_id)

    _BACKEND_IMAGE_STATUS_TO_IMAGE_BUILD_STATUS = {
        ClusterEnvironmentBuildStatus.PENDING: ImageBuildStatus.IN_PROGRESS,
        ClusterEnvironmentBuildStatus.IN_PROGRESS: ImageBuildStatus.IN_PROGRESS,
        ClusterEnvironmentBuildStatus.SUCCEEDED: ImageBuildStatus.SUCCEEDED,
        ClusterEnvironmentBuildStatus.FAILED: ImageBuildStatus.FAILED,
        ClusterEnvironmentBuildStatus.PENDING_CANCELLATION: ImageBuildStatus.FAILED,
        ClusterEnvironmentBuildStatus.CANCELED: ImageBuildStatus.FAILED,
    }

    def _get_image_build_status(
        self, build: ClusterEnvironmentBuild
    ) -> ImageBuildStatus:
        return cast(
            ImageBuildStatus,
            self._BACKEND_IMAGE_STATUS_TO_IMAGE_BUILD_STATUS.get(
                build.status, ImageBuildStatus.UNKNOWN
            ),
        )

    def get(self, name: str) -> ImageBuild:
        """Get an image by name with optional version (name:version format).

        If no version is specified, returns the latest build.
        If a version is specified, searches for that specific revision.
        """
        image_name, version_str = name.split(":", 1) if ":" in name else (name, None)
        version: Optional[int] = None
        if version_str:
            try:
                version = int(version_str)
            except ValueError:
                raise ValueError(
                    f"Invalid version format '{version_str}'. Version must be a number."
                )

        response = self.client.list_application_templates(
            name=image_name,
            image_name=None,
            creator_id=None,
            project=None,
            include_archived=False,
            defaults_first=False,
            count=50,
            paging_token=None,
        )

        # list_application_templates does substring matching, need exact match
        results = response.results if response.results else []
        template = next((t for t in results if t.name == image_name), None)
        if not template:
            raise ValueError(f"Image '{image_name}' not found.")

        if not template.latest_build:
            raise ValueError(f"No builds found for image '{image_name}'.")

        # If specific version requested, verify it exists and get its details
        if version is not None and template.latest_build.revision != version:
            for b in self.client.list_cluster_env_builds(template.id):
                if b.revision == version and b.id:
                    image_uri_obj = self.get_image_uri_from_build_id(
                        b.id, use_image_alias=True
                    )
                    return ImageBuild(
                        id=template.id or "",
                        name=template.name or "",
                        project_id=template.project_id,
                        creator_id=template.creator_id,
                        creator_email=template.creator.email
                        if template.creator
                        else None,
                        is_anonymous=bool(template.anonymous),
                        created_at=template.created_at,
                        last_modified_at=template.last_modified_at,
                        latest_build_id=b.id,
                        latest_build_revision=b.revision,
                        latest_build_status=self._get_image_build_status(b),
                        latest_image_uri=image_uri_obj.image_uri
                        if image_uri_obj
                        else None,
                    )
            raise ValueError(f"Version {version} not found for image '{image_name}'.")

        return self._application_template_to_image_build(template)

    def build_image_from_containerfile(
        self,
        name: str,
        containerfile: str,
        ray_version: Optional[str] = None,
        anonymous: bool = False,
    ) -> str:
        return self.client.get_cluster_env_build_id_from_containerfile(
            cluster_env_name=name,
            containerfile=containerfile,
            anonymous=anonymous,
            ray_version=ray_version,
        )

    def build_image_from_containerfile_with_image_uri(
        self,
        name: str,
        containerfile: str,
        ray_version: Optional[str] = None,
        anonymous: bool = False,
    ) -> str:
        build_id = self.build_image_from_containerfile(
            name=name,
            containerfile=containerfile,
            ray_version=ray_version,
            anonymous=anonymous,
        )
        image_uri = self.get_image_uri_from_build_id(build_id)
        if image_uri:
            return image_uri.image_uri
        raise RuntimeError(
            f"This is a bug! Failed to get image uri for build {build_id} that just created."
        )

    def build_image_from_requirements(
        self, name: str, base_build_id: str, requirements: List[str]
    ):
        if requirements:
            base_build = self.client.get_cluster_env_build(base_build_id)
            if (
                base_build
                and base_build.status == ClusterEnvironmentBuildStatus.SUCCEEDED
                and base_build.docker_image_name
            ):
                self.logger.info(f"Using tracked python packages: {requirements}")
                lines = [
                    "# syntax=docker/dockerfile:1",
                    f"FROM {base_build.docker_image_name}",
                ]
                for requirement in requirements:
                    lines.append(f'RUN pip install "{requirement}"')
                return self.build_image_from_containerfile(
                    name, "\n".join(lines), ray_version=base_build.ray_version
                )
            else:
                raise RuntimeError(
                    f"Base build {base_build_id} is not a successful build."
                )
        else:
            return base_build_id

    def registery_image(
        self,
        image_uri: str,
        registry_login_secret: Optional[str] = None,
        ray_version: Optional[str] = None,
    ) -> str:
        image_uri_checked = ImageURI.from_str(image_uri_str=image_uri)
        return self.client.get_cluster_env_build_id_from_image_uri(
            image_uri=image_uri_checked,
            registry_login_secret=registry_login_secret,
            ray_version=ray_version,
        )

    def register_byod_image_with_name(
        self,
        image_uri_str: str,
        name: str,
        ray_version: Optional[str] = None,
        registry_login_secret: Optional[str] = None,
    ) -> str:
        image_uri_checked = ImageURI.from_str(image_uri_str=image_uri_str)
        if image_uri_checked.is_cluster_env_image():
            raise RuntimeError(
                f"Image URI {image_uri_str} is not a BYOD image. "
                "The 'register' command only works with BYOD images."
            )

        build_id = self.client.get_cluster_env_build_id_from_image_uri(
            image_uri=image_uri_checked,
            registry_login_secret=registry_login_secret,
            ray_version=ray_version,
            name=name,
        )

        fetched_image_uri = self.get_image_uri_from_build_id(
            build_id, use_image_alias=True
        )
        if fetched_image_uri:
            return fetched_image_uri.image_uri
        raise RuntimeError(
            f"This is a bug! Failed to get image uri for build {build_id} that just created."
        )

    def get_image_uri_from_build_id(
        self, build_id: str, use_image_alias: bool = False
    ) -> Optional[ImageURI]:
        return self.client.get_cluster_env_build_image_uri(
            cluster_env_build_id=build_id, use_image_alias=use_image_alias,
        )

    def _application_template_to_image_build(
        self, template: DecoratedApplicationTemplate
    ) -> ImageBuild:
        """Convert a DecoratedApplicationTemplate to an ImageBuild."""
        latest_build = template.latest_build
        latest_build_id = latest_build.id if latest_build else None
        latest_build_revision = latest_build.revision if latest_build else None
        latest_build_status = None
        latest_image_uri = None

        if latest_build:
            if latest_build.status:
                with suppress(ValueError):
                    latest_build_status = ImageBuildStatus.validate(
                        latest_build.status.upper()
                    )

            latest_image_uri = latest_build.docker_image_name
            if not latest_image_uri and latest_build_id:
                image_uri = self.get_image_uri_from_build_id(
                    latest_build_id, use_image_alias=True
                )
                if image_uri:
                    latest_image_uri = image_uri.image_uri

        creator_email = template.creator.email if template.creator else None

        return ImageBuild(
            id=template.id or "",
            name=template.name or "",
            project_id=template.project_id,
            creator_id=template.creator_id,
            creator_email=creator_email,
            is_anonymous=bool(template.anonymous),
            created_at=template.created_at,
            last_modified_at=template.last_modified_at,
            latest_build_id=latest_build_id,
            latest_build_revision=latest_build_revision,
            latest_build_status=latest_build_status,
            latest_image_uri=latest_image_uri,
        )

    def _should_filter_template(
        self,
        template: DecoratedApplicationTemplate,
        include_anonymous: bool,
        include_archived: bool,
    ) -> bool:
        """Return True if template should be filtered out."""
        if not include_anonymous and template.anonymous:
            return True
        if not include_archived and template.archived_at is not None:
            return True
        return False

    def list(  # noqa: PLR0913, PLR0917
        self,
        *,
        image_id: Optional[str] = None,
        name: Optional[str] = None,
        image_name: Optional[str] = None,
        project: Optional[str] = None,
        creator_id: Optional[str] = None,
        include_archived: bool = False,
        include_anonymous: bool = False,
        max_items: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> ResultIterator[ImageBuild]:
        """List images or fetch a single image by ID."""
        if page_size is not None and not (1 <= page_size <= MAX_PAGE_SIZE):
            raise ValueError(
                f"page_size must be between 1 and {MAX_PAGE_SIZE}, inclusive."
            )

        # Handle single image lookup by ID
        if image_id is not None:
            template = self.client.get_application_template(image_id)

            if template is None or self._should_filter_template(
                template, include_anonymous, include_archived
            ):
                # Return empty iterator
                return ResultIterator(
                    page_token=None,
                    max_items=0,
                    fetch_page=lambda _: SimpleNamespace(
                        results=[], metadata=SimpleNamespace(next_paging_token=None),
                    ),
                    parse_fn=None,
                )

            # Return single-item iterator
            summary = self._application_template_to_image_build(template)
            return ResultIterator(
                page_token=None,
                max_items=1,
                fetch_page=lambda token: SimpleNamespace(
                    results=[summary] if token is None else [],
                    metadata=SimpleNamespace(next_paging_token=None),
                ),
                parse_fn=None,
            )

        # Handle paginated list
        def _fetch_page(token: Optional[str]) -> SimpleNamespace:
            response = self.client.list_application_templates(
                name=name,
                image_name=image_name,
                creator_id=creator_id,
                project=project,
                include_archived=include_archived,
                defaults_first=False,
                count=page_size,
                paging_token=token,
            )

            results = response.results if response.results else []
            filtered_results = [
                template
                for template in results
                if not self._should_filter_template(
                    template, include_anonymous, include_archived
                )
            ]

            next_token = (
                response.metadata.next_paging_token if response.metadata else None
            )

            return SimpleNamespace(
                results=filtered_results,
                metadata=SimpleNamespace(next_paging_token=next_token),
            )

        return ResultIterator(
            page_token=None,
            max_items=max_items,
            fetch_page=_fetch_page,
            parse_fn=self._application_template_to_image_build,
        )



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class JobReport(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'job_id': 'str',
        'unused_cpu_hours': 'float',
        'unused_gpu_hours': 'float',
        'max_instances_launched': 'int'
    }

    attribute_map = {
        'job_id': 'job_id',
        'unused_cpu_hours': 'unused_cpu_hours',
        'unused_gpu_hours': 'unused_gpu_hours',
        'max_instances_launched': 'max_instances_launched'
    }

    def __init__(self, job_id=None, unused_cpu_hours=None, unused_gpu_hours=None, max_instances_launched=None, local_vars_configuration=None):  # noqa: E501
        """JobReport - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._job_id = None
        self._unused_cpu_hours = None
        self._unused_gpu_hours = None
        self._max_instances_launched = None
        self.discriminator = None

        self.job_id = job_id
        if unused_cpu_hours is not None:
            self.unused_cpu_hours = unused_cpu_hours
        if unused_gpu_hours is not None:
            self.unused_gpu_hours = unused_gpu_hours
        if max_instances_launched is not None:
            self.max_instances_launched = max_instances_launched

    @property
    def job_id(self):
        """Gets the job_id of this JobReport.  # noqa: E501


        :return: The job_id of this JobReport.  # noqa: E501
        :rtype: str
        """
        return self._job_id

    @job_id.setter
    def job_id(self, job_id):
        """Sets the job_id of this JobReport.


        :param job_id: The job_id of this JobReport.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and job_id is None:  # noqa: E501
            raise ValueError("Invalid value for `job_id`, must not be `None`")  # noqa: E501

        self._job_id = job_id

    @property
    def unused_cpu_hours(self):
        """Gets the unused_cpu_hours of this JobReport.  # noqa: E501


        :return: The unused_cpu_hours of this JobReport.  # noqa: E501
        :rtype: float
        """
        return self._unused_cpu_hours

    @unused_cpu_hours.setter
    def unused_cpu_hours(self, unused_cpu_hours):
        """Sets the unused_cpu_hours of this JobReport.


        :param unused_cpu_hours: The unused_cpu_hours of this JobReport.  # noqa: E501
        :type: float
        """

        self._unused_cpu_hours = unused_cpu_hours

    @property
    def unused_gpu_hours(self):
        """Gets the unused_gpu_hours of this JobReport.  # noqa: E501


        :return: The unused_gpu_hours of this JobReport.  # noqa: E501
        :rtype: float
        """
        return self._unused_gpu_hours

    @unused_gpu_hours.setter
    def unused_gpu_hours(self, unused_gpu_hours):
        """Sets the unused_gpu_hours of this JobReport.


        :param unused_gpu_hours: The unused_gpu_hours of this JobReport.  # noqa: E501
        :type: float
        """

        self._unused_gpu_hours = unused_gpu_hours

    @property
    def max_instances_launched(self):
        """Gets the max_instances_launched of this JobReport.  # noqa: E501


        :return: The max_instances_launched of this JobReport.  # noqa: E501
        :rtype: int
        """
        return self._max_instances_launched

    @max_instances_launched.setter
    def max_instances_launched(self, max_instances_launched):
        """Sets the max_instances_launched of this JobReport.


        :param max_instances_launched: The max_instances_launched of this JobReport.  # noqa: E501
        :type: int
        """

        self._max_instances_launched = max_instances_launched

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, JobReport):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, JobReport):
            return True

        return self.to_dict() != other.to_dict()



"""
Defines and implements `anyscale list {resource}` commands.
Supported resources: clouds, projects, sessions, ips
"""

from typing import Any, Optional

import click

from anyscale.cli_logger import BlockLogger
from anyscale.commands.util import DeprecatedAnyscaleCommand
from anyscale.controllers.list_controller import ListController


log = BlockLogger()


@click.group(
    "list", hidden=True, help="List resources (projects, clusters) within Anyscale."
)
def list_cli() -> None:
    pass


@list_cli.command(
    name="clouds",
    help="[DEPRECATED] List the clouds currently available in your account.",
    hidden=True,
    cls=DeprecatedAnyscaleCommand,
    removal_date="2025-10-01",
    deprecation_message="`anyscale list clouds` has been deprecated",
    alternative="use `anyscale cloud list` instead",
)
@click.option("--json", "show_json", help="Return the results in json", is_flag=True)
def list_clouds(show_json: bool) -> None:
    """List the clouds currently available in your account.

    DEPRECATED: This command will be removed on 2025-10-01.
    Use 'anyscale cloud list' instead.
    """
    list_controller = ListController()
    output = list_controller.list_clouds(json_format=show_json)
    print(output)


@list_cli.command(
    name="projects",
    help="[DEPRECATED] List all accessible projects.",
    hidden=True,
    cls=DeprecatedAnyscaleCommand,
    removal_date="2025-10-01",
    deprecation_message="`anyscale list projects` has been deprecated",
    alternative="use `anyscale project list` instead",
)
@click.option("--json", "show_json", help="Return the results in json", is_flag=True)
@click.pass_context
def project_list(ctx: Any, show_json: bool) -> None:  # noqa: ARG001
    """List all accessible projects.

    DEPRECATED: This command will be removed on 2025-10-01.
    Use 'anyscale project list' instead.
    """
    list_controller = ListController()
    output = list_controller.list_projects(json_format=show_json)
    print(output)


@list_cli.command(
    name="sessions",
    help="[DEPRECATED] List all clusters within the current project.",
    hidden=True,
    cls=DeprecatedAnyscaleCommand,
    removal_date="2025-10-01",
    deprecation_message="`anyscale list sessions` has been deprecated",
    alternative="use `anyscale cluster list` instead",
)
@click.option(
    "--name",
    help="Name of the cluster. If provided, this prints the snapshots that "
    "were applied and commands that ran for all clusters that match "
    "this name.",
    default=None,
)
@click.option("--all", help="List all clusters, including inactive ones.", is_flag=True)
@click.option("--json", "show_json", help="Return the results in json", is_flag=True)
def session_list(name: Optional[str], all: bool, show_json: bool) -> None:  # noqa: A002
    """List all clusters within the current project.

    DEPRECATED: This command will be removed on 2025-10-01.
    Use 'anyscale cluster list' instead.
    """
    list_controller = ListController()
    output = list_controller.list_sessions(
        name=name, show_all=all, json_format=show_json
    )
    print(output)


@list_cli.command(
    name="ips",
    help="[DEPRECATED] List IP addresses of head and worker nodes.",
    context_settings={"ignore_unknown_options": True, "allow_extra_args": True,},
    cls=DeprecatedAnyscaleCommand,
    removal_date="2025-10-01",
    deprecation_message="Listing IPs is not supported on Anyscale",
    alternative="use the Anyscale console to view cluster node information",
)
def list_ips() -> None:
    """List IP addresses of head and worker nodes.

    DEPRECATED: This command will be removed on 2025-10-01.
    Listing IPs is not supported on Anyscale.
    Use the Anyscale console to view cluster node information.
    """
    raise click.ClickException("Listing IPs is not supported on Anyscale")
